#! /usr/bin/python3
# -*- coding: utf-8 -*-

from xml.dom import minidom
import codecs
import sqlite3


# Le nom du fichier xml source
# ~ FICHIER_TELLICO = "mini.xml"
FICHIER_TELLICO = "Le bottin des jeux Linux.tc"

# Le nom de la base SQLite3 de destination
BASE_DE_DONNEES = 'bottin.db'

# Champs possibles pour une fiche
FieldsAttributes = ["category", "type", "flags", "title", "name", "format", "description","allowed" ]

# On ne s'intéresse qu'aux fiches ?
ONLY_ENTRY = True

# Instructions SQL
# ================
NULL = 'NULL'

# Collection
SQL_DROP_COLLECTION   = """drop table if exists Collection"""
SQL_CREATE_COLLECTION = """
create table Collection (
      id Integer PRIMARY KEY
    , title char
    , type Integer
)"""
SQL_INSERT_COLLECTION   ="""insert into Collection (title, type) values ('%s', '%s')"""
SQL_DELETE_COLLECTION   ="""delete from Collection where id = %s"""

# Entry
SQL_DROP_ENTRY   = """drop table if exists Entry"""
SQL_CREATE_ENTRY = """
create table Entry (
      "id" integer primary key
    , "binaire-disponible" text
    , "catégorie" text
    , "cdate" text
    , "comments" text
    , "commercial" text
    , "cover" text
    , "demo" text
    , "description" text
    , "description-en" text
    , "description-new" text
    , "extrepo" text
    , "fiche-créée-par" text
    , "fonctionnalités" text
    , "genre" text
    , "graphic" text
    , "installation" text
    , "level-of-finition-of-this-record" text
    , "lgv" text
    , "licences" text
    , "links2" text
    , "mdate" text
    , "mechanics" text
    , "paquet-debianubuntu-disponible" text
    , "portss" text
    , "préférences-de-la-période" text
    , "rating" text
    , "repository" text
    , "réserve" text
    , "reserved" text
    , "source-disponible-" text
    , "sous-catégorie-news" text
    , "status" text
    , "stockpile" text
    , "summary-en" text
    , "title" text
    , "type" text
    , "version" text
    , "video" text
    , "view" text
    , "why-" text
)"""
SQL_INSERT_ENTRY   ="""insert into entry (%s) values (%s)"""
SQL_DELETE_ENTRY   ="""delete from entry where id = %s"""

# Vue simplifiée de la base
SQL_CREATE_VIEW ="""
create view vEntry as
select "id", "title" as Nom, "genre" as Genre, "catégorie" as Category, 
replace(replace(replace(replace(replace(replace(replace(`sous-catégorie-news`,char(0x0a),""),"  ",""),"sous-catégorie-new",""),"<>",";"),"<s>;",""),"</>",""),"</s>","") as Tags, 
"mechanics" as Mechanics, "stockpile" as Played, "description-new" as "Fr", "summary-en" as "En" from Entry
"""
SQL_DROP_VIEW = """drop view if exists vEntry"""

# quelques constantes propres à minidom
TEXT_NODE = minidom.Node.TEXT_NODE
ELEMENT_NODE = minidom.Node.ELEMENT_NODE



# Classes
# ===================

class Db:
    """Tout ce qui concerne les accès à la base Sqlite3"""
    
    def __init__(self, basename = BASE_DE_DONNEES):
        self.db = sqlite3.connect(basename)
        self.cursor = self.db.cursor()
        self.cursor.execute( SQL_DROP_COLLECTION )
        self.cursor.execute( SQL_DROP_ENTRY )
        self.cursor.execute( SQL_DROP_VIEW )
        self.cursor.execute( SQL_CREATE_COLLECTION )        
        self.cursor.execute( SQL_CREATE_ENTRY )
        self.cursor.execute( SQL_CREATE_VIEW )

    def ajouterCollection(self, title='', type_collection=''):
        self.cursor.execute(SQL_INSERT_COLLECTION % (title, type_collection))

    def ajouterFiche(self, fiche):
        # Impossible que l'id ne soit pas connu lors de la lecture du Tellico
        if 'id' in fiche:
            # ~ print(SQL_DELETE_ENTRY % fiche['id'])
            self.cursor.execute(SQL_DELETE_ENTRY % fiche['id'])
            columns = ""
            values = ""
            for c,v in fiche.items():
                columns = columns + ',"' + c + '"'
                values = values + ',"' + v.replace('"', '""') + '"'
            # retrait de la première virgule
            columns = columns[1:]
            values = values[1:]
            # ~ print(SQL_INSERT_ENTRY % (columns, values))
            self.cursor.execute( SQL_INSERT_ENTRY % (columns, values))

    def commit(self):
        self.db.commit()

    def fermerBase(self):
        self.db.commit()
        self.db.close()

class FichierTellico:
    """Une classe permettant le décodage des fichiers Tellico
    """
    
    def __init__(self):
        """ Ouverture de la base SQLite3 et du fichier Tellico"""
        self.base = Db(BASE_DE_DONNEES)
        self.doc = minidom.parse(FICHIER_TELLICO)
        
        self.collection = self.doc.getElementsByTagName("collection")[0]
        self.title = self.collection.getAttribute("title")
        self.type = self.collection.getAttribute("type")
        # On ajoute la collection à la base SQLite
        self.base.ajouterCollection(self.title, self.type)
        
    
    def terminer(self):
        self.base.fermerBase()

    def afficherInfoTellico(self):
        print("\n\n\n")
        print('=================================================================')
        print('Décodage de la collection "%s" de type %s' % (self.title, self.type))
        print('=================================================================')
        
    def afficherChampsPossibles(self):
        print("\n\n")
        print('=================================================================')
        print('Décodage des champs possibles')
        print('=================================================================')

        fields  = self.collection.getElementsByTagName("fields")[0] # pointe sur <fields>
        fields  = fields.getElementsByTagName("field") # liste des fields
        for field in fields:
            print("----")
            # liste des attributs
            for name in FieldsAttributes:
                if field.hasAttribute(name):
                    print('%s : "%s"' % (name, field.getAttribute(name)))
            # il peut y avoir une liste de "prop" avec des attributs 
            if field.hasChildNodes():
                for prop in field.getElementsByTagName("prop"):
                    print('   prop = { name : "%s", value : "%s"}' % (prop.getAttribute("name"), prop.firstChild.nodeValue))
            
    def analyserFiches(self):
        """methode permettant de décoder toutes les fiches et de les écrire
           dans la base SQLite3.
        """
        for entry in self.collection.getElementsByTagName("entry"):
            id_entry = entry.getAttribute("id")
            fiche = {"id" : id_entry}
            print("Entry id=%s" % id_entry)
            if entry.hasChildNodes():
                e = entry.firstChild
                while e != None:
                    if e.nodeType == ELEMENT_NODE:
                        if e.hasAttribute("calendar"):
                            value = e.getAttribute("calendar")               
                        else:
                            value = e.firstChild.nodeValue.strip()
                        name =  e.nodeName
                        
                        # ~ print('{ name : "%s", value : "%s"}' % (name, value))
                        if e.hasChildNodes():
                            childValue = e.toxml().strip()
                            # ~ print(childValue)
                            d = e.firstChild
                            while d != None:
                                if d.nodeType == ELEMENT_NODE:
                                    value = childValue
                                    # ~ print('    { name : "%s", value : "%s"}' % (d.nodeName, d.firstChild.nodeValue.strip()))
                                d = d.nextSibling
                        fiche.update( {name : value} )
                    e = e.nextSibling
                #print(fiche)
                self.base.ajouterFiche(fiche)


# Programme principal
# ===================

fichier = FichierTellico()


fichier.afficherInfoTellico()
# ~ fichier.afficherChampsPossibles()
fichier.analyserFiches()
fichier.terminer()

print("ok")

